@@ -1,3 +1,4 @@
+from Acquisition import aq_base
 from DateTime import DateTime
 from Products.CMFPlone.interfaces import IPloneSiteRoot
 from Products.Five import BrowserView
@@ -7,12 +8,16 @@ from ftw.publisher.core.interfaces import IDataCollector
 from ftw.publisher.receiver import decoder
 from ftw.publisher.receiver import getLogger
 from ftw.publisher.receiver.events import AfterCreatedEvent, AfterUpdatedEvent
+from plone.app.uuid.utils import uuidToObject
 from zope import event
 from zope.component import getAdapters
+from zope.event import notify
+from zope.lifecycleevent import ObjectAddedEvent
 from zope.publisher.interfaces import Retry
 import os.path
 import sys
 import traceback
+import plone.uuid
 
 
 class ReceiveObject(BrowserView):
@@ -39,7 +44,9 @@ class ReceiveObject(BrowserView):
                 # otherwise we encapsulate the exception in a UnexpectedError
                 exc = ''.join(traceback.format_exception(*sys.exc_info()))
                 state = states.UnexpectedError(exc)
+
             self.logger.error('request failed: %s' % (state.toString()))
+
         # get the string representation of the CommunicationState
         resp = communication.createResponse(state)
         try:
@@ -47,6 +54,8 @@ class ReceiveObject(BrowserView):
             resp += ''.join(traceback.format_exception(*sys.exc_info()))
         except:
             pass
+
+        self.request.RESPONSE.setHeader('X-Theme-Disabled', 'True')
         return resp
 
     def handleRequest(self):
@@ -63,27 +72,34 @@ class ReceiveObject(BrowserView):
             'Receiving request (data length: %i Byte)' % len(jsondata))
         if not jsondata:
             raise states.InvalidRequestError('No jsondata provided')
+
         # decode the jsondata to a python dictionary
-        #XXX we do this now twice (also in core adapters)
+        # XXX we do this now twice (also in core adapters)
         self.decoder = decoder.Decoder(self.context)
         self.data = self.decoder(jsondata)
+
         # get the action type ..
         action = self.getAction()
+
         # .. and run the action specific method ..
-        if action=='push':
+        if action == 'push':
             # extract medata from data, we want to pass metadata
             # separatly to dataCollector adapters
             metadata = self.data['metadata']
             return self.pushAction(metadata, self.data)
+
         elif action == 'move':
             metadata = self.data['metadata']
             return self.moveAction(metadata, self.data)
-        elif action=='delete':
+
+        elif action == 'delete':
             metadata = self.data['metadata']
             return self.deleteAction(metadata)
+
         else:
             # ... or raise a UnexpectedError
             raise states.UnknownActionError()
+
         # should not get here
         raise states.UnexpectedError()
 
@@ -140,14 +156,14 @@ class ReceiveObject(BrowserView):
 
                     # if we can't fix it we will raise the followin exception
                     exception = states.UIDPathMismatchError({
-                            'problem': 'path wrong',
-                            'found path': current_path,
-                            'expected path': expected_path,
-                            })
+                        'problem': 'path wrong',
+                        'found path': current_path,
+                        'expected path': expected_path,
+                    })
 
                     # is it in the same place? -> rename
                     if current_path.split('/')[:-1] == \
-                            expected_path.split('/')[:-1]:
+                       expected_path.split('/')[:-1]:
                         putils = obj1.plone_utils
                         success, failure = putils.renameObjectsByPaths(
                             (current_path, ), (expected_path.split('/')[-1], ),
@@ -174,14 +190,12 @@ class ReceiveObject(BrowserView):
                         else:
                             object = self._getObjectByPath(expected_path)
 
-
                 elif object.UID() != metadata['UID']:
                     raise states.UIDPathMismatchError({
-                            'problem': 'UID wrong',
-                            'found uid': object.UID(),
-                            'expected uid': metadata['UID'],
-                            })
-
+                        'problem': 'UID wrong',
+                        'found uid': object.UID(),
+                        'expected uid': metadata['UID'],
+                    })
 
         parent_modified_date = None
 
@@ -189,10 +203,10 @@ class ReceiveObject(BrowserView):
         new_object = False
         if not object:
             self.logger.info(
-                'Object with UID %s does not existing: creating new object'%(
+                'Object with UID %s does not existing: creating new object' % (
                     metadata['UID'],
-                    )
                 )
+            )
             # ... find container
             container = self._findContainerObjectByPath(absPath)
             try:
@@ -203,17 +217,28 @@ class ReceiveObject(BrowserView):
                 raise states.ErrorState('Could not find container of %s' %
                                         absPath)
             self.logger.info('... container: "%s" at %s' % (
-                    container.Title(),
-                    '/'.join(container.getPhysicalPath()),
-                    ))
+                container.Title(),
+                '/'.join(container.getPhysicalPath()),
+            ))
             # ... create object
             object = container.get(container.invokeFactory(
-                    metadata['portal_type'],
-                    metadata['id'],
-                    ))
-            object._setUID(metadata['UID'])
-            #object.processForm()
+                metadata['portal_type'],
+                metadata['id'],
+            ))
+
+            if hasattr(aq_base(object), '_setUID'):
+                object._setUID(metadata['UID'])
+
+            else:
+                setattr(object,
+                        plone.uuid.interfaces.ATTRIBUTE_NAME,
+                        metadata['UID'])
+
+                notify(ObjectAddedEvent(object))
+
+            # object.processForm()
             new_object = True
+
         else:
             try:
                 parent_modified_date = object.aq_inner.aq_parent.modified()
@@ -221,7 +246,7 @@ class ReceiveObject(BrowserView):
                 pass
 
         # finalize
-        if not is_root:
+        if hasattr(aq_base(object), 'processForm'):
             object.processForm()
 
         if not is_root:
@@ -243,7 +268,7 @@ class ReceiveObject(BrowserView):
                 wf.updateRoleMappingsFor(object)
 
         # updates all data with the registered adapters for IDataCollector
-        adapters = getAdapters((object, ), IDataCollector)
+        adapters = sorted(getAdapters((object, ), IDataCollector))
         for name, adapter in adapters:
             data = self.decoder.unserializeFields(object, name)
             adapter.setData(data[name], metadata)
@@ -326,7 +351,7 @@ class ReceiveObject(BrowserView):
                         failure.get(obj_path).__str__()))
 
         else:
-            #object has been moved
+            # object has been moved
             portal_path = '/'.join(
                 self.context.portal_url.getPortalObject().getPhysicalPath())
             old_parent_path = portal_path + move_data['oldParent']
@@ -354,7 +379,6 @@ class ReceiveObject(BrowserView):
                         obj_path,
                         'Target parent does not exist, source object deleted'))
 
-
         # return a ObjectMovedState() instance
         return states.ObjectMovedState()
 
@@ -388,7 +412,7 @@ class ReceiveObject(BrowserView):
         @type uid:      string
         @return:        Plone-Object or None
         """
-        return self.context.reference_catalog.lookupObject(uid)
+        return uuidToObject(uid)
 
     def _getObjectByPath(self, absolutePath):
         """
@@ -402,17 +426,17 @@ class ReceiveObject(BrowserView):
         # plone site root is not in catalog ...
         portalObject = self.context.portal_url.getPortalObject()
         portalPath = '/'.join(portalObject.getPhysicalPath())
-        if absolutePath==portalPath:
+        if absolutePath == portalPath:
             # plone site root is searched, return it
             return portalObject
         # for any other object we use the catalog tool
         brains = self.context.portal_catalog({
-                'path': {
-                    'query': absolutePath,
-                    'depth': 0,
-                    },
-                })
-        if len(brains)==0:
+            'path': {
+                'query': absolutePath,
+                'depth': 0,
+            },
+        })
+        if len(brains) == 0:
             return None
         else:
             return brains[0].getObject()
@@ -469,7 +493,7 @@ class ReceiveObject(BrowserView):
         """
         positions = metadata['sibling_positions']
         parent = object.aq_inner.aq_parent
-        object_ids = [o['id'] for o in parent._objects]
+        object_ids = list(parent.objectIds())
 
         # move objects with no position info to the bottom
         for id in object_ids:
@@ -482,7 +506,6 @@ class ReceiveObject(BrowserView):
         # order objects
         parent.moveObjectsByDelta(object_ids, -len(object_ids))
 
-
         # reindex all objects
         for id in object_ids:
             try:
