@@ -1,13 +1,26 @@
+from AccessControl.SecurityInfo import ClassSecurityInformation
+from Acquisition import aq_base
 from Products.Archetypes.Field import FileField
 from Products.Archetypes.Field import ImageField
 from Products.Archetypes.Field import ReferenceField
 from Products.CMFPlone.interfaces import IPloneSiteRoot
+from StringIO import StringIO
 from ftw.publisher.core import states
 from ftw.publisher.core.utils import encode_after_json
 from ftw.publisher.receiver import getLogger
-from zope.component import queryAdapter
 import base64
-import simplejson
+import json
+import pkg_resources
+
+try:
+    pkg_resources.get_distribution('plone.app.blob')
+
+except pkg_resources.DistributionNotFound:
+    HAS_BLOBS = False
+
+else:
+    HAS_BLOBS = True
+    from plone.app.blob.interfaces import IBlobField
 
 
 class Decoder(object):
@@ -16,6 +29,8 @@ class Decoder(object):
     It also validates and decodes all schema field values.
     """
 
+    security = ClassSecurityInformation()
+
     def __init__(self, context):
         """
         Constructor: stores context as object attribute
@@ -25,6 +40,7 @@ class Decoder(object):
         self.context = context
         self.logger = getLogger()
 
+    security.declarePrivate('__call__')
     def __call__(self, jsondata):
         """
         Decodes the jsondata to a dictionary, validates it,
@@ -36,10 +52,11 @@ class Decoder(object):
         self.validate()
         return self.data
 
+    security.declarePrivate('decodeJson')
     def decodeJson(self, jsondata):
         """
-        Decodes the JSON data with the simplejson module.
-        If the simplejson module cannot decode the string, a
+        Decodes the JSON data with the json module.
+        If the json module cannot decode the string, a
         DecodeError is raised.
         @param jsondata:    JSON data
         @type jsondata:     string
@@ -48,12 +65,13 @@ class Decoder(object):
         @raise:             DecodeError
         """
         try:
-            data = simplejson.loads(jsondata)
+            data = json.loads(jsondata)
         except Exception, e:
             raise states.DecodeError(str(e))
         data = encode_after_json(data)
         return data
 
+    security.declarePrivate('validate')
     def validate(self):
         """
         Validates, if all required values are provided. If a
@@ -62,7 +80,7 @@ class Decoder(object):
         @return:            None
         """
         structure = {
-            'metadata' : [
+            'metadata': [
                 'UID',
                 'id',
                 'portal_type',
@@ -77,8 +95,10 @@ class Decoder(object):
             else:
                 for subkey in structure[key]:
                     if subkey not in self.data[key]:
-                        raise states.PartialError('Missing "%s.%s"' % (key, subkey))
+                        raise states.PartialError(
+                            'Missing "%s.%s"' % (key, subkey))
 
+    security.declarePrivate('getSchema')
     def getSchema(self, object):
         """
         Returns the Schema of the portal_type defined in the metadata.
@@ -98,7 +118,8 @@ class Decoder(object):
         else:
             return None
 
-    def unserializeFields(self,object, jsonkey):
+    security.declarePrivate('unserializeFields')
+    def unserializeFields(self, object, jsonkey):
         """
         Unserializes the fielddata and optimizes it of the modifiers of
         the fields.
@@ -107,7 +128,14 @@ class Decoder(object):
 
         # we just have to go throught if we have a dict, with possible
         # schema fields
-        if not isinstance(self.data[jsonkey],dict):
+        if not isinstance(self.data[jsonkey], dict):
+            return self.data
+
+        if IPloneSiteRoot.providedBy(object):
+            return self.data
+
+        if not hasattr(aq_base(object), 'Schema'):
+            # might be dexterity
             return self.data
 
         fields = object.Schema().fields()
@@ -116,21 +144,27 @@ class Decoder(object):
             name = field.getName()
             if name not in self.data[jsonkey].keys():
                 continue
+
             # DateTimeField doesnt need to be converted t DateTime
             # FileFields are base64 encoded
-            if isinstance(field, FileField):
+
+            if HAS_BLOBS and IBlobField.providedBy(field) or \
+               isinstance(field, (ImageField, FileField)):
                 value = self.data[jsonkey][name]
-                if isinstance(value, dict):
+
+                if isinstance(value, dict) and not value['data']:
+                    self.data[jsonkey][name] = None
+
+                elif isinstance(value, dict):
                     # decode it
-                    data = base64.decodestring(value['data'])
-                    filename = value['filename']
-                    # process it
-                    file, mimetype, filename = field._process_input(data, filename=filename)
-                    # we only use the file object
-                    self.data[jsonkey][name] = file
+                    data = StringIO(base64.decodestring(value['data']))
+                    data.seek(0)
+                    setattr(data, 'filename', value['filename'])
+                    self.data[jsonkey][name] = data
+
             # ReferenceField: remove bad UIDs
             if isinstance(field, ReferenceField):
-                #check if field ist multiValued
+                # check if field ist multiValued
                 if field.multiValued:
                     cleaned = []
                     for uid in self.data[jsonkey][name]:
@@ -139,25 +173,40 @@ class Decoder(object):
                             cleaned.append(uid)
                         else:
                             if uid:
-                                self.logger.warn("The reference field <%s> of object(%s) has a broken reference to the object %s" % (name,object.UID(),uid))
+                                self.logger.warn(
+                                    ('The reference field <%s> of object(%s)'
+                                     ' has a broken reference to'
+                                     ' the object %s') % (name,
+                                                          object.UID(),
+                                                          uid))
+
                     self.data[jsonkey][name] = cleaned
+
                 else:
                     cleaned = None
-                    obj = self.context.reference_catalog.lookupObject(self.data[jsonkey][name])
+                    obj = self.context.reference_catalog.lookupObject(
+                        self.data[jsonkey][name])
                     if obj:
                         cleaned = self.data[jsonkey][name]
                     else:
                         if self.data[jsonkey][name]:
-                            self.logger.warn("The reference field <%s> of object(%s) has a broken reference to the object %s" % (name,object.UID(),self.data[jsonkey][name]))
+                            self.logger.warn(
+                                ('The reference field <%s> of object(%s) has a'
+                                 ' broken reference to the object %s') % (
+                                     name,
+                                     object.UID(),
+                                     self.data[jsonkey][name]))
+
                     self.data[jsonkey][name] = cleaned
 
             # ImageField: treat empty files special
             if isinstance(field, ImageField):
-                if not self.data[jsonkey][name] or len(self.data[jsonkey][name])==0:
+                if not self.data[jsonkey][name]:
                     self.data[jsonkey][name] = 'DELETE_IMAGE'
+
             # FileField (direct): treat empty files special
-            if field.__class__==FileField:
-                if not self.data[jsonkey][name] or len(self.data[jsonkey][name])==0:
+            if field.__class__ == FileField:
+                if not self.data[jsonkey][name]:
                     self.data[jsonkey][name] = 'DELETE_FILE'
 
         return self.data
